(eval-when-compile
  (require 'cl))

(require 'spamfilter)
(require 'gnus-art)
(require 'gnus-sum)
(require 'gnus-score)


;;; variables
(defvar spamf-gnus-corpus-file (expand-file-name "~/.spamf-corpus")
  "File name of corpus.")

(defvar spamf-gnus-spam-score -100
  "Score bias for articles judged as spam.")


;;; utility functions.
(defun spamf-gnus-prepare (article)
  (save-excursion
    (set-buffer nntp-server-buffer)
    (gnus-request-article article gnus-newsgroup-name)
    (run-hooks 'gnus-article-decode-hook)))

(defun spamf-gnus-register-spam-buffer (article)
  (spamf-gnus-prepare article)
  (spamf-register-spam-buffer nntp-server-buffer))

(defun spamf-gnus-register-good-buffer (article)
  (spamf-gnus-prepare article)
  (spamf-register-good-buffer nntp-server-buffer))

(defun spamf-gnus-score-article (article)
  (spamf-gnus-prepare article)
  (when (spamf-spam-buffer-p nntp-server-buffer)
    (let ((score (assq article gnus-newsgroup-scored)))
      (if score (setcdr score (+ (cdr score) spamf-gnus-spam-score))
	(push (cons article spamf-gnus-spam-score) gnus-newsgroup-scored)))))

(defun spamf-gnus-apply-filter ()
  (dolist (header gnus-newsgroup-headers)
    (let ((n (mail-header-number header)))
      (when (not (assq n gnus-newsgroup-scored))
        (message "scoring article %d" n)
        (spamf-gnus-score-article n))))
  (message ""))


;;; interactive functions.
(defun spamf-gnus-register-deleted-as-spam ()
  "Registers deleted articles as spam."
  (interactive)
  (dolist (article gnus-newsgroup-data)
    (when (not (gnus-data-pseudo-p article))
      (let ((m (gnus-data-mark article))
	    (n (gnus-data-number article)))
	(when (= m gnus-del-mark)
	  (message "bad article %d" n)
	  (spamf-gnus-register-spam-buffer n)))))
  (message ""))

(defun spamf-gnus-register-processable-as-good ()
  "Registers processable articles as good."
  (interactive)
  (dolist (article gnus-newsgroup-processable)
    (message "good article %d" article)
    (spamf-gnus-register-good-buffer article))
  (message ""))

(defun spamf-gnus-register-marked ()
  "Register marked articles as spam / good according to mark."
  (interactive)
  (spamf-gnus-register-deleted-as-spam)
  (spamf-gnus-register-processable-as-good))

(defun spamf-gnus-save-corpus ()
  (interactive)
  (message "saving corpus....")
  (spamf-save-corpus spamf-gnus-corpus-file)
  (message "saving corpus.... done."))

(defun spamf-gnus-load-corpus ()
  (interactive)
  (message "loading corpus....")
  (spamf-load-corpus spamf-gnus-corpus-file)
  (message "loading corpus.... done."))


;;; advices.
(defadvice gnus-score-headers (after spamf-gnus-score-headers activate)
  "Advice for scoring with spamfilter."
  (spamf-gnus-apply-filter))
